@@ -24,8 +24,8 @@
 #define SEQ24_GLOBALS
 
 #include <string>
-#include <gtk--/main.h>
-#include <gtk--/drawingarea.h>
+#include <gtkmm/main.h>
+#include <gtkmm/drawingarea.h>
 
 using namespace std;
 
@@ -89,9 +89,9 @@ const int c_keyoffset_x = c_keyarea_x - c_key_x;
 const int c_rollarea_y = c_keyarea_y;
 
 /* events bar */
-const int c_eventarea_y = 20;
-const int c_eventevent_y = 14;
-const int c_eventevent_x = 7;
+const int c_eventarea_y = 16;
+const int c_eventevent_y = 10;
+const int c_eventevent_x = 5;
 
 /* time scale window on top */
 const int c_timearea_y = 18;
@@ -101,7 +101,7 @@ const int c_midi_notes = 256;
 const string c_dummy( "Untitled" );
 
 /* maximum size of sequence, default size */
-const int c_maxbeats     = 64;   /* max number of beats in a sequence */
+const int c_maxbeats     = 0xFFFF;   /* max number of beats in a sequence */
 
 
 /* midifile tags */
@@ -120,29 +120,32 @@ const unsigned long c_midictrl =   0x24240010;
 const string 
 c_about( 
 
-	string ( PACKAGE ) + string( " " ) + string( VERSION ) +
-	string ( " Interactive Midi Sequencer\n" )+
+	string ( PACKAGE ) + string( " " ) + string( VERSION ) + string( " " ) +
+	string ( "Interactive Midi Sequencer\n" )+
 	string ( "\n" )+
-	string ( "Author: Rob C. Buse (rcbuse@filter24.org)\n" )+
+	string ( "Author: Rob C. Buse (seq24@filter24.org)\n" )+
 	string ( "\n" )+
 	string ( "For information about seq24, read the SEQ24\n" )+
 	string ( "file included with the source.\n" )+
 	string ( "\n" )+
-	string ( "Visit http://www.filter24.org for updates.\n" )+
+	string ( "Visit http://www.filter24.org/seq24 for updates.\n" )+
 	string ( "\n" )+
-	string ( "Copyright (c) 2003  Rob C. Buse\n" )+
+	string ( "Copyright (c) 2005  Rob C. Buse\n" )+
 	string ( "Released under the GNU Public License, see\n" )+
 	string ( "COPYING for details.\n" )
 );
 
-extern Gdk_Font c_text_font_6_12;
-extern Gdk_Font c_text_font_8_13;
-extern Gdk_Font c_text_font_5_7;
+const char c_font_6_12[] = "-*-fixed-medium-r-*--12-*-*-*-*-*-*";
+const char c_font_8_13[] = "-*-fixed-medium-r-*--13-*-*-*-*-*-*";
+const char c_font_5_7[]  = "-*-fixed-medium-r-*--7-*-*-*-*-*-*";
+
+
+
 
 /* used in menu to tell setState what to do */
 const int c_adding = 0;
 const int c_normal = 1;
-const int c_paste = 2;
+const int c_paste  = 2;
 
 /* redraw when recording ms */
 const int c_redraw_ms = 40;
@@ -158,6 +161,14 @@ extern bool global_showmidi;
 extern bool global_priority;
 extern bool global_stats;
 extern bool global_pass_sysex;
+extern bool global_with_jack_transport;
+extern bool global_with_jack_master;
+extern bool global_with_jack_master_cond;
+extern bool global_jack_start_mode;
+
+extern std::string global_filename;
+
+extern bool global_print_keys;
 
 /* scales */
 enum c_music_scales {
@@ -184,6 +195,51 @@ const bool c_scales_policy[c_scale_size][12] =
 
   };
 
+const int c_scales_transpose_up[c_scale_size][12] =
+
+  {
+    /* off = chromatic */
+    { 1,1,1,1,1,1,1,1,1,1,1,1},
+    /* major */
+    { 2,0,2,0,1,2,0,2,0,2,0,1},
+    /* minor */
+    { 2,0,1,2,0,2,0,1,2,0,2,0},
+
+  };
+
+
+
+
+const int c_scales_transpose_dn[c_scale_size][12] =
+
+  {
+    /* off = chromatic */
+    { -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1},
+    /* major */
+    { -1,0,-2,0,-2,-1,0,-2,0,-2,0,-2},
+    /* minor */
+    { -2,0,-2,-1,0,-2,0,-2,-1,0,-2,0},
+
+  };
+
+const int c_scales_symbol[c_scale_size][12] =
+
+  {
+    /* off = chromatic */
+    { 32,32,32,32,32,32,32,32,32,32,32,32},
+
+    /* major */
+    { 32,32,32,32,32,32,32,32,32,32,32,32},
+               
+    /* minor */
+    { 32,32,32,32,32,32,32,32,129,128,129,128},
+                 
+  };             
+                 
+// up 128        
+// down 129
+
+
 const char c_scales_text[c_scale_size][20] =
   { 
     "Off", 
@@ -194,20 +250,56 @@ const char c_scales_text[c_scale_size][20] =
 const char c_key_text[][20] =
   {
     "C", 
-    "C#/Db", 
+    "C#", 
     "D", 
-    "Eb", 
+    "D#", 
     "E",
     "F", 
-    "F#/Gb",
+    "F#",
     "G",
-    "Ab",
+    "G#",
     "A",
-    "Bb",
-    "B/Cb"
+    "A#",
+    "B"
   };
 
-//const char 
+const char c_interval_text[][15] =
+{
+    "P1",
+    "m2",
+    "M2",
+    "m3",
+    "M3",
+    "P4",
+    "TT",
+    "P5",
+    "m6",
+    "M6",
+    "m7",
+    "M7",
+    "P8",
+    "m9",
+    "M9",
+    ""
+
+};
+ 	  	  	
+
+
+const char c_chord_text[][8] =
+{
+    "I",
+    "II",
+    "III",
+    "IV",
+    "V",
+    "VI",
+    "VII",
+    "VIII"
+};
+
+
+ 
 
 
 #endif
