@@ -34,12 +34,12 @@ class maintime: public Gtk::DrawingArea
 
  private: 
 
-    Gdk_GC       m_gc;
-    Gdk_Window   m_window;
-    Gdk_Color    m_black, m_white, m_grey;
+    Glib::RefPtr<Gdk::GC>       m_gc;
+    Glib::RefPtr<Gdk::Window>   m_window;
+    Gdk::Color    m_black, m_white, m_grey;
 
-    void realize_impl();
-    int expose_event_impl(GdkEventExpose* a_ev);
+    void on_realize();
+    bool on_expose_event(GdkEventExpose* a_ev);
 
     long m_tick;
 
