@@ -20,6 +20,7 @@
 
 #include "mainwid.h"
 #include "seqedit.h"
+#include "font.h"
 
 
 const char mainwid::m_seq_to_char[c_seqs_in_set] =
@@ -44,65 +45,138 @@ mainwid::mainwid( perform *a_p  ): DrawingArea()
 
     m_window_x = c_mainwid_x;
     m_window_y = c_mainwid_y;
+    
+    Glib::RefPtr<Gdk::Colormap> colormap = get_default_colormap();
 
-    Gdk_Colormap colormap = get_default_colormap();
+    m_black = Gdk::Color( "black" );
+    m_white = Gdk::Color( "white" );
+    m_grey  = Gdk::Color( "grey" );
 
-    m_black = Gdk_Color( "black" );
-    m_white = Gdk_Color( "white" );
-    m_grey  = Gdk_Color( "grey" );
+    colormap->alloc_color( m_black );
+    colormap->alloc_color( m_white );
+    colormap->alloc_color( m_grey );
 
-    colormap.alloc( m_black );
-    colormap.alloc( m_white );
-    colormap.alloc( m_grey );
-
-    size ( c_mainwid_x, c_mainwid_y );
+    set_size_request( c_mainwid_x, c_mainwid_y );
   
-    add_events( GDK_BUTTON_PRESS_MASK | 
-		GDK_BUTTON_RELEASE_MASK |
-		GDK_KEY_PRESS_MASK |
-		GDK_BUTTON_MOTION_MASK |
-		GDK_FOCUS_CHANGE_MASK  );
+    add_events( Gdk::BUTTON_PRESS_MASK | 
+		Gdk::BUTTON_RELEASE_MASK |
+		Gdk::KEY_PRESS_MASK |
+		Gdk::BUTTON_MOTION_MASK |
+		Gdk::FOCUS_CHANGE_MASK  );
   
-    m_menu = manage( new Menu());
-
     m_screenset = 0;
 
-    m_menu->items().push_back(MenuElem("New/Edit", slot(this,&mainwid::seq_edit)));
-    m_menu->items().push_back(SeparatorElem());
-    m_menu->items().push_back(MenuElem("Copy", slot(this,&mainwid::seq_copy)));
-    m_menu->items().push_back(MenuElem("Cut", slot(this,&mainwid::seq_cut)));
-    m_menu->items().push_back(MenuElem("Paste", slot(this,&mainwid::seq_paste)));
-    m_menu->items().push_back(SeparatorElem());
-    m_menu->items().push_back(MenuElem("Clear Performance Data", slot(this,&mainwid::seq_clear_perf)));
- 
-     m_button_down = false;
+  
+    m_button_down = false;
     m_moving = false;
 
+    m_menu = NULL;
+
+     set_double_buffered( false );
+
 } 
 
 
+void
+mainwid::popup_menu( void )
+{
+
+    using namespace Menu_Helpers;
+    
+    if ( m_menu != NULL )
+        delete m_menu;
+    
+    m_menu = manage( new Menu());
+
+    if ( m_mainperf->is_active( m_current_seq )) {
+        m_menu->items().push_back(MenuElem("Edit", slot(*this,&mainwid::seq_edit)));
+    } else {
+        m_menu->items().push_back(MenuElem("New", slot(*this,&mainwid::seq_edit)));
+    }
+    
+
+        
+    m_menu->items().push_back(SeparatorElem());
+
+    if ( m_mainperf->is_active( m_current_seq )) {
+        m_menu->items().push_back(MenuElem("Cut", slot(*this,&mainwid::seq_cut)));
+        m_menu->items().push_back(MenuElem("Copy", slot(*this,&mainwid::seq_copy)));
+    } else {
+        m_menu->items().push_back(MenuElem("Paste", slot(*this,&mainwid::seq_paste)));
+    }
+    
+    if ( m_mainperf->is_active( m_current_seq )) {
+    
+        m_menu->items().push_back(SeparatorElem());
+        m_menu->items().push_back(MenuElem("Clear Song Data", slot(*this,&mainwid::seq_clear_perf)));
+        m_menu->items().push_back(SeparatorElem());
+
+        Menu *menu_buses = manage( new Menu() );
+         
+        m_menu->items().push_back( MenuElem( "Midi Bus", *menu_buses) );
+        
+        /* midi buses */
+        mastermidibus *masterbus = m_mainperf->get_master_midi_bus();
+        for ( int i=0; i< masterbus->get_num_out_buses(); i++ ){
+
+            Menu *menu_channels = manage( new Menu() );
+            
+            menu_buses->items().push_back(MenuElem( masterbus->get_midi_bus_name(i),
+                                                    *menu_channels ));
+            char b[4];
+            
+            /* midi channel menu */
+            for( int j=0; j<16; j++ ){
+                
+                sprintf( b, "%d", j+1 );
+                menu_channels->items().push_back(MenuElem(b, 
+                                                          bind(slot(*this,&mainwid::set_bus_and_midi_channel), 
+                                                               i, j )));
+            }
+        }        
+    }
+    
+    m_menu->popup(0,0);
+
+}
+
+void
+mainwid::set_bus_and_midi_channel( int a_bus, int a_ch )
+{
+    if ( m_mainperf->is_active( m_current_seq )) {
+        m_mainperf->get_sequence( m_current_seq )->set_midi_bus( a_bus );
+        m_mainperf->get_sequence( m_current_seq )->set_midi_channel( a_ch );
+
+        draw_sequence_on_pixmap( m_current_seq  );
+        draw_sequence_pixmap_on_window( m_current_seq );
+    }
+}
+
+
 
 // Menu callback, Lanches Editor Window
 void 
 mainwid::seq_edit(){
 
+    seqedit *seq_edit;
+    
     if ( m_mainperf->is_active( m_current_seq )) {
-	if ( !m_mainperf->get_sequence( m_current_seq )->get_editing()){
-	    manage( new seqedit( m_mainperf->get_sequence( m_current_seq ), 
-				 m_mainperf, 
-				 this,
-				 m_current_seq
-				 ));
-	}
+        if ( !m_mainperf->get_sequence( m_current_seq )->get_editing()){
+            seq_edit = new seqedit( m_mainperf->get_sequence( m_current_seq ), 
+                                    m_mainperf, 
+                                    this,
+                                    m_current_seq
+                                    );
+        }
     }
     else {
-	this->seq_new();
-	manage( new seqedit( m_mainperf->get_sequence( m_current_seq ), 
-			     m_mainperf, 
-			     this, 
-			     m_current_seq
-			     ));
-    }
+        this->seq_new();
+        seq_edit = new seqedit( m_mainperf->get_sequence( m_current_seq ), 
+                                m_mainperf, 
+                                this, 
+                                m_current_seq
+                                );
+    }    
 }
 
 // Makes a New sequence 
@@ -113,7 +187,7 @@ mainwid::seq_new(){
 
 	m_mainperf->new_sequence( m_current_seq );
 	draw_sequence_on_pixmap( m_current_seq  );
-	draw_pixmap_on_window();
+	draw_sequence_pixmap_on_window( m_current_seq );
     }
 }
 
@@ -135,7 +209,7 @@ mainwid::seq_cut(){
 	m_clipboard = *(m_mainperf->get_sequence( m_current_seq ));
 	m_mainperf->delete_sequence( m_current_seq );
 	draw_sequence_on_pixmap( m_current_seq  );
-	draw_pixmap_on_window();
+	draw_sequence_pixmap_on_window( m_current_seq );
     }
 }
 
@@ -149,7 +223,7 @@ mainwid::seq_paste(){
 	*(m_mainperf->get_sequence( m_current_seq )) = m_clipboard;
 	
 	draw_sequence_on_pixmap( m_current_seq  );
-	draw_pixmap_on_window();
+	draw_sequence_pixmap_on_window( m_current_seq );
     }
 }
 
@@ -165,25 +239,30 @@ mainwid::seq_clear_perf(){
 
 // GTK, on realize of window, init shiz
 void 
-mainwid::realize_impl()
+mainwid::on_realize()
 {
     // we need to do the default realize
-    Gtk::DrawingArea::realize_impl();
+    Gtk::DrawingArea::on_realize();
 
-    set_flags( GTK_CAN_FOCUS );
+    set_flags( Gtk::CAN_FOCUS );
 
     // Now we can allocate any additional resources we need
     m_window = get_window();
-    m_gc.create( m_window );
-    m_window.clear();
+    m_gc = Gdk::GC::create( m_window );
+    m_window->clear();
 
-    m_pixmap = Gdk_Pixmap(m_window, 
-			  c_mainwid_x, 
-			  c_mainwid_y );
+    p_font_renderer->init( m_window );
+
+    m_pixmap = Gdk::Pixmap::create(m_window, 
+                                   c_mainwid_x, 
+                                   c_mainwid_y,
+                                   -1 );
 
     fill_background_window();
     draw_sequences_on_pixmap();
 
+    
+
 }
 
 // Fills Pixmap
@@ -204,12 +283,12 @@ mainwid::fill_background_window()
 {
     /* clear background */
 
-    m_pixmap.draw_rectangle(this->get_style()->get_bg_gc(GTK_STATE_NORMAL),
-			    true,
-			    0,
-			    0, 
-			    m_window_x, 
-			    m_window_y );
+    m_pixmap->draw_rectangle(this->get_style()->get_bg_gc(Gtk::STATE_NORMAL),
+    		    true,
+    		    0,
+    		    0, 
+    		    m_window_x, 
+    		    m_window_y );
 }
 
 int
@@ -237,8 +316,8 @@ mainwid::draw_sequence_on_pixmap( int a_seq )
 
 	int local_seq = a_seq % c_seqs_in_set;
 
-	m_gc.set_foreground(m_black);
-	m_pixmap.draw_rectangle(m_gc, true,
+	m_gc->set_foreground(m_black);
+	m_pixmap->draw_rectangle(m_gc, true,
 				base_x,
 				base_y,
 				c_seqarea_x,  
@@ -261,28 +340,32 @@ mainwid::draw_sequence_on_pixmap( int a_seq )
 	      m_foreground = m_black;
 	    }
 
-		if ( seq->get_queued() ){
-	      
-			m_background = m_grey;
-			m_foreground = m_black;
-		}
-	
-	    m_gc.set_foreground(m_background);
-	    m_pixmap.draw_rectangle(m_gc,true,
+            m_gc->set_foreground(m_background);
+	    m_pixmap->draw_rectangle(m_gc,true,
 				    base_x + 1,
 				    base_y + 1,
 				    c_seqarea_x - 2,  
 				    c_seqarea_y - 2 );
 	
 	    
-	    m_gc.set_foreground(m_foreground);
+	    m_gc->set_foreground(m_foreground);
 	    char name[20];
 	    sprintf( name, "%.13s", seq->get_name() );
 
-	    m_pixmap.draw_string(c_text_font_6_12, m_gc, 
-				 base_x + c_text_x, 
-				 base_y + c_text_y,
-				 name );
+
+        font::Color col = font::BLACK;;
+        
+        if ( m_foreground == m_black ){
+            col = font::BLACK;
+        }
+        if ( m_foreground == m_white ){
+            col = font::WHITE;
+        }
+
+        p_font_renderer->render_string_on_drawable( m_gc, 
+                                                    base_x + c_text_x, 
+                                                    base_y + 4,
+                                                    m_pixmap, name, col);
 	
 	    /* midi channel + key + timesig */
 
@@ -290,29 +373,43 @@ mainwid::draw_sequence_on_pixmap( int a_seq )
 	  
 	    char str[20];
 	    sprintf( str, 
-		     "%c  %1d-%02d %2d/%2d", key, 
+		     "%1d-%02d    %2d/%2d", 
 		     seq->get_midi_bus(), 
 		     seq->get_midi_channel()+1,
 		     seq->get_bpm(), seq->get_bw() );
-
-	    
-	    m_pixmap.draw_string(c_text_font_6_12, m_gc, 
-				 base_x + c_text_x, 
-				 base_y + c_text_y * 5 - c_text_y / 2,
-				 str );
+        
+	    p_font_renderer->render_string_on_drawable(m_gc,
+                                                   base_x + c_text_x, 
+                                                   base_y + c_text_y * 4 - 2,
+                                                   m_pixmap, str, col );
 
     
 
     
-	    int rectangle_x = base_x + c_text_x;
-	    int rectangle_y = base_y + c_text_y + c_text_x;
-    
-	    m_gc.set_foreground(m_foreground);
-	    m_pixmap.draw_rectangle(m_gc,false,
-				    rectangle_x - 2,
-				    rectangle_y - 1,
-				    c_seqarea_seq_x + 3,  
-				    c_seqarea_seq_y + 3 );
+	    int rectangle_x = base_x + c_text_x - 1;
+	    int rectangle_y = base_y + c_text_y + c_text_x - 1;
+
+            if ( seq->get_queued() ){
+                
+                m_gc->set_foreground(m_grey);
+                m_pixmap->draw_rectangle(m_gc,true,
+                                         rectangle_x - 2,
+                                         rectangle_y - 1,
+                                         c_seqarea_seq_x + 3,  
+                                         c_seqarea_seq_y + 3 );
+
+                 m_foreground = m_black;
+                
+            }
+
+
+            
+	    m_gc->set_foreground(m_foreground);
+	    m_pixmap->draw_rectangle(m_gc,false,
+                                     rectangle_x - 2,
+                                     rectangle_y - 1,
+                                     c_seqarea_seq_x + 3,  
+                                     c_seqarea_seq_y + 3 );
     
 	    int lowest_note = seq->get_lowest_note_event( );
 	    int highest_note = seq->get_highest_note_event( );
@@ -348,8 +445,8 @@ mainwid::draw_sequence_on_pixmap( int a_seq )
 		if ( tick_f_x <= tick_s_x )
 		    tick_f_x = tick_s_x + 1; 
     
-		m_gc.set_foreground(m_foreground);
-		m_pixmap.draw_line(m_gc, rectangle_x + tick_s_x,
+		m_gc->set_foreground(m_foreground);
+		m_pixmap->draw_line(m_gc, rectangle_x + tick_s_x,
 					 rectangle_y + note_y,
 					 rectangle_x + tick_f_x,  
 					 rectangle_y + note_y );
@@ -361,20 +458,47 @@ mainwid::draw_sequence_on_pixmap( int a_seq )
     
 	    /* not active */
     
-	    m_gc.set_foreground(m_grey);
-	    m_pixmap.draw_rectangle(this->get_style()->get_bg_gc(GTK_STATE_NORMAL),
+	    m_gc->set_foreground(m_grey);
+	    m_pixmap->draw_rectangle( this->get_style()->get_bg_gc(Gtk::STATE_NORMAL),
 				    true,
 				    base_x + 4,       base_y,
 				    c_seqarea_x - 8,  c_seqarea_y );
     
-	    m_pixmap.draw_rectangle(this->get_style()->get_bg_gc(GTK_STATE_NORMAL),
+	    m_pixmap->draw_rectangle( this->get_style()->get_bg_gc(Gtk::STATE_NORMAL),
 				    true,
 				    base_x + 1,       base_y + 1,
-				    c_seqarea_x - 2,  c_seqarea_y - 2 );
+				    c_seqarea_x - 2,  c_seqarea_y - 2 ); 
 	}
     }
 }
 
+void
+mainwid::draw_sequence_pixmap_on_window( int a_seq )
+{
+    if ( a_seq >= (m_screenset  * c_mainwnd_rows * c_mainwnd_cols ) &&
+         a_seq <  ((m_screenset+1)  * c_mainwnd_rows * c_mainwnd_cols )){
+        
+        int i =  (a_seq / c_mainwnd_rows) % c_mainwnd_cols;
+        int j =  a_seq % c_mainwnd_rows;
+        
+        int base_x = (c_mainwid_border +
+                      (c_seqarea_x + c_mainwid_spacing) * i);
+        int base_y = (c_mainwid_border +
+                      (c_seqarea_y + c_mainwid_spacing) * j);
+
+        m_window->draw_drawable(m_gc, 
+                                m_pixmap, 
+                                base_x,
+                                base_y,
+                                base_x,
+                                base_y,
+                                c_seqarea_x,
+                                c_seqarea_y );    }
+	    
+
+    
+}
+
   
 void 
 mainwid::update_markers( int a_ticks )
@@ -404,8 +528,8 @@ mainwid::draw_marker_on_sequence( int a_seq, int a_tick )
 	int base_y = (c_mainwid_border +
 		  (c_seqarea_y + c_mainwid_spacing) * j);
 
-	int rectangle_x = base_x + c_text_x;
-	int rectangle_y = base_y + c_text_y + c_text_x;
+	int rectangle_x = base_x + c_text_x - 1;
+	int rectangle_y = base_y + c_text_y + c_text_x - 1;
 	
 	int length = seq->get_length( );
 	a_tick %= length;
@@ -413,7 +537,7 @@ mainwid::draw_marker_on_sequence( int a_seq, int a_tick )
 	long tick_x = a_tick * c_seqarea_seq_x / length;
 
 	   
-        m_window.draw_pixmap(m_gc, 
+        m_window->draw_drawable(m_gc, 
     			 m_pixmap, 
     			 rectangle_x + m_last_tick_x[a_seq],
 			 rectangle_y + 1,
@@ -425,17 +549,17 @@ mainwid::draw_marker_on_sequence( int a_seq, int a_tick )
 	m_last_tick_x[a_seq] = tick_x;
 
 	if ( seq->get_playing() ){
-		m_gc.set_foreground(m_white);    
+		m_gc->set_foreground(m_white);    
 	} else {
-		m_gc.set_foreground(m_black); 
+		m_gc->set_foreground(m_black); 
 	}
 
 	if ( seq->get_queued()){
-		m_gc.set_foreground(m_black);
+		m_gc->set_foreground(m_black);
 	}
 
 
-	m_window.draw_line(m_gc,
+	m_window->draw_line(m_gc,
 			   rectangle_x + tick_x,
 			   rectangle_y + 1,
 			   rectangle_x + tick_x,  
@@ -458,7 +582,7 @@ void
 mainwid::update_sequence_on_window( int a_seq   )
 {
     draw_sequence_on_pixmap( a_seq );
-    draw_pixmap_on_window();
+    draw_sequence_pixmap_on_window( a_seq );
 }
 
 // queues blit of pixmap to window
@@ -470,10 +594,10 @@ mainwid::draw_pixmap_on_window()
 
 
 // GTK expose event
-int 
-mainwid::expose_event_impl(GdkEventExpose* a_e)
+bool
+mainwid::on_expose_event(GdkEventExpose* a_e)
 {
-    m_window.draw_pixmap(m_gc, 
+    m_window->draw_drawable(m_gc, 
     			 m_pixmap, 
     			 a_e->area.x,
 			 a_e->area.y,
@@ -525,8 +649,8 @@ mainwid::seq_from_xy( int a_x, int a_y )
  
 
 // press a mouse button
-int 
-mainwid::button_press_event_impl(GdkEventButton* a_p0)
+bool
+mainwid::on_button_press_event(GdkEventButton* a_p0)
 {
 
     grab_focus();
@@ -535,9 +659,7 @@ mainwid::button_press_event_impl(GdkEventButton* a_p0)
 
     // launch menu (right button)
     if (  m_current_seq != -1 && a_p0->button == 3  ){
- 
-        GdkEventButton *bevent = (GdkEventButton *) a_p0;
-        m_menu->popup(bevent->button, bevent->time);
+        popup_menu();
     }
 
     if ( m_current_seq != -1  && a_p0->button == 1 ){
@@ -549,8 +671,8 @@ mainwid::button_press_event_impl(GdkEventButton* a_p0)
 }
 
 
-int 
-mainwid::button_release_event_impl(GdkEventButton* a_p0)
+bool
+mainwid::on_button_release_event(GdkEventButton* a_p0)
 {
 
     m_current_seq = seq_from_xy( (int) a_p0->x, (int) a_p0->y );
@@ -570,7 +692,7 @@ mainwid::button_release_event_impl(GdkEventButton* a_p0)
 		m_mainperf->sequence_playing_toggle( m_current_seq );
 
 	    draw_sequence_on_pixmap(  m_current_seq );
-	    draw_pixmap_on_window();
+	    draw_sequence_pixmap_on_window( m_current_seq);
 	}
     }
 
@@ -585,7 +707,7 @@ mainwid::button_release_event_impl(GdkEventButton* a_p0)
 	    *(m_mainperf->get_sequence( m_current_seq )) = m_moving_seq;
 	
 	    draw_sequence_on_pixmap( m_current_seq  );
-	    draw_pixmap_on_window();
+	    draw_sequence_pixmap_on_window( m_current_seq );
 
 	} else {
 
@@ -593,7 +715,7 @@ mainwid::button_release_event_impl(GdkEventButton* a_p0)
 	    *(m_mainperf->get_sequence( m_old_seq )) = m_moving_seq;
 	
 	    draw_sequence_on_pixmap( m_old_seq  );
-	    draw_pixmap_on_window();
+	    draw_sequence_pixmap_on_window( m_old_seq );
 	}
 
 
@@ -602,8 +724,8 @@ mainwid::button_release_event_impl(GdkEventButton* a_p0)
     return true;
 }
 
-int 
-mainwid::motion_notify_event_impl(GdkEventMotion* a_p0)
+bool
+mainwid::on_motion_notify_event(GdkEventMotion* a_p0)
 {
     int seq = seq_from_xy( (int) a_p0->x, (int) a_p0->y );
 
@@ -620,7 +742,7 @@ mainwid::motion_notify_event_impl(GdkEventMotion* a_p0)
 		m_moving_seq = *(m_mainperf->get_sequence( m_current_seq ));
 		m_mainperf->delete_sequence( m_current_seq );
 		draw_sequence_on_pixmap( m_current_seq  );
-		draw_pixmap_on_window();
+		draw_sequence_pixmap_on_window( m_current_seq );
 	    }
 	}
     }
@@ -639,11 +761,11 @@ mainwid::reset( )
 }
 
 
-int 
-mainwid::get_screenset( )
-{
-    return m_screenset;
-}
+//int 
+//mainwid::get_screenset( )
+//{
+//    return m_screenset;
+//}
 
 void 
 mainwid::set_screenset( int a_ss )
@@ -671,20 +793,16 @@ mainwid::~mainwid( )
 
 
 
-int 
-mainwid::focus_in_event_impl(GdkEventFocus*)
+bool
+mainwid::on_focus_in_event(GdkEventFocus*)
 {
-    set_flags(GTK_HAS_FOCUS);
-
-    draw_focus( );
+    set_flags(Gtk::HAS_FOCUS);
     return false;
 }
 
-int 
-mainwid::focus_out_event_impl(GdkEventFocus*)
+bool
+mainwid::on_focus_out_event(GdkEventFocus*)
 {
-    unset_flags(GTK_HAS_FOCUS);
-    draw_focus();
-
+    unset_flags(Gtk::HAS_FOCUS);
     return false;
 }
