@@ -19,32 +19,37 @@
 //-----------------------------------------------------------------------------
 
 
+
+#include "globals.h"
+#include "perform.h"
+
+class seqedit;
+
 #ifndef SEQ24_MAINWID
 #define SEQ24_MAINWID
 
 
-#include <gtk--/button.h>
-#include <gtk--/window.h>
-#include <gtk--/accelgroup.h>
-#include <gtk--/box.h>
-#include <gtk--/main.h>
-#include <gtk--/menu.h>
-#include <gtk--/menubar.h>
-#include <gtk--/eventbox.h>
-#include <gtk--/window.h>
-#include <gtk--/table.h>
-#include <gtk--/drawingarea.h>
-#include <gtk--/widget.h>
-#include <gtk--/style.h> 
+#include <gtkmm/button.h>
+#include <gtkmm/window.h>
+#include <gtkmm/accelgroup.h>
+#include <gtkmm/box.h>
+#include <gtkmm/main.h>
+#include <gtkmm/menu.h>
+#include <gtkmm/menubar.h>
+#include <gtkmm/eventbox.h>
+#include <gtkmm/window.h>
+#include <gtkmm/table.h>
+#include <gtkmm/drawingarea.h>
+#include <gtkmm/widget.h>
+#include <gtkmm/style.h> 
+
 
-#include "globals.h"
-#include "perform.h"
 
 using namespace Gtk;
 using SigC::slot;
 
 const int c_seqarea_seq_x = c_text_x * 13;
-const int c_seqarea_seq_y = c_text_y * 2 - 2;
+const int c_seqarea_seq_y = c_text_y * 2;
 
 /* piano roll */
 class mainwid : public Gtk::DrawingArea
@@ -52,16 +57,16 @@ class mainwid : public Gtk::DrawingArea
 
  private: 
 
-    Gdk_GC       m_gc;
-    Gdk_Window   m_window;
-    Gdk_Color    m_black, m_white, m_grey, m_dk_grey;
-    Gdk_Color    m_background, m_foreground;
+    Glib::RefPtr<Gdk::GC> m_gc;
+    Glib::RefPtr<Gdk::Window> m_window;
+    Gdk::Color   m_black, m_white, m_grey, m_dk_grey;
+    Gdk::Color   m_background, m_foreground;
 
-    Gdk_Pixmap   m_pixmap;
+    Glib::RefPtr<Gdk::Pixmap>   m_pixmap;
    
     GdkRectangle m_old;
     GdkRectangle m_selected;
-
+    
     int          m_screenset;
 
     Menu         *m_menu;
@@ -92,16 +97,14 @@ class mainwid : public Gtk::DrawingArea
     bool m_last_playing[c_max_sequence];
     static const char m_seq_to_char[c_seqs_in_set];
 
+    void on_realize();
 
-
-    void realize_impl();
-
-    int expose_event_impl(GdkEventExpose* a_ev);
-    int button_press_event_impl(GdkEventButton* a_ev); 
-    int button_release_event_impl(GdkEventButton* a_ev);
-    int motion_notify_event_impl(GdkEventMotion* a_p0);
-    int focus_in_event_impl(GdkEventFocus*);
-    int focus_out_event_impl(GdkEventFocus*);
+    bool on_expose_event(GdkEventExpose* a_ev);
+    bool on_button_press_event(GdkEventButton* a_ev); 
+    bool on_button_release_event(GdkEventButton* a_ev);
+    bool on_motion_notify_event(GdkEventMotion* a_p0);
+    bool on_focus_in_event(GdkEventFocus*);
+    bool on_focus_out_event(GdkEventFocus*);
 
     void seq_edit();
     void seq_new();
@@ -110,18 +113,25 @@ class mainwid : public Gtk::DrawingArea
     void seq_cut();
     void seq_paste(); 
 
-	void seq_clear_perf();
+    void seq_clear_perf();
+
+    void set_bus_and_midi_channel( int a_bus, int a_ch );
+
+    void popup_menu();
  
     void draw_sequence_on_pixmap( int a_seq );
     void draw_sequences_on_pixmap();
 
     void fill_background_window();
     void draw_pixmap_on_window();
+    void draw_sequence_pixmap_on_window( int a_seq );
 
     int seq_from_xy( int a_x, int a_y );
 
     int timeout( void );
 
+ 
+
  public:
 
     mainwid( perform *a_p );
@@ -129,7 +139,7 @@ class mainwid : public Gtk::DrawingArea
 
     void reset();
  
-    int get_screenset( ); 
+    //int get_screenset( ); 
     void set_screenset( int a_ss );
 
     void update_sequence_on_window( int a_seq  );
