@@ -18,36 +18,46 @@
 //
 //-----------------------------------------------------------------------------
 
-#ifndef SEQ24_MAINWINDOW
-#define SEQ24_MAINWINDOW
-
-#include <gtk--/button.h>
-#include <gtk--/window.h>
-#include <gtk--/accelgroup.h>
-#include <gtk--/box.h>
-#include <gtk--/main.h>
-#include <gtk--/menu.h>
-#include <gtk--/menubar.h>
-#include <gtk--/eventbox.h>
-#include <gtk--/window.h>
-#include <gtk--/table.h>
-#include <gtk--/drawingarea.h>
-#include <gtk--/label.h>
-#include <gtk--/viewport.h> 
-#include <gtk--/fileselection.h>
-#include <gtk--/dialog.h>
-#include <gtk--/text.h>
-#include <gtk--/arrow.h>
-#include <gtk--/spinbutton.h>
-#include <gtk--/progressbar.h>
-
 
+#include "mainwid.h"
 #include "perform.h"
 #include "sequence.h"
 #include "event.h"
-#include "mainwid.h"
 #include "options.h"
 #include "maintime.h"
+#include "perfedit.h"
+#include "options.h"
+
+
+
+#ifndef SEQ24_MAINWINDOW
+#define SEQ24_MAINWINDOW
+
+#include <gtkmm/button.h>
+#include <gtkmm/window.h>
+#include <gtkmm/accelgroup.h>
+#include <gtkmm/box.h>
+#include <gtkmm/main.h>
+#include <gtkmm/menu.h>
+#include <gtkmm/menubar.h>
+#include <gtkmm/eventbox.h>
+#include <gtkmm/window.h>
+#include <gtkmm/table.h>
+#include <gtkmm/drawingarea.h>
+#include <gtkmm/label.h>
+#include <gtkmm/viewport.h> 
+#include <gtkmm/fileselection.h>
+#include <gtkmm/dialog.h>
+#include <gtkmm/messagedialog.h>
+#include <gtkmm/textview.h>
+#include <gtkmm/arrow.h>
+#include <gtkmm/spinbutton.h>
+#include <gtkmm/progressbar.h>
+#include <gtkmm/dialog.h>
+#include <gtkmm/label.h>
+#include <gdkmm/cursor.h>
+#include <gtkmm/image.h>
+
 
 #include "globals.h"
 
@@ -64,7 +74,9 @@ class mainwnd : public Gtk::Window
 {
    
  private:
- 
+
+    bool      m_quit;
+    
     MenuBar  *m_menubar;
     Menu     *m_menu_file;
     Menu     *m_menu_control;
@@ -75,12 +87,14 @@ class mainwnd : public Gtk::Window
     mainwid  *m_main_wid;
     maintime *m_main_time;
 
-    Gdk_Cursor   m_main_cursor;
-    
-    FileSelection *m_filesel;
+    perfedit *m_perf_edit;
+    options *m_options;
 
+    Gdk::Cursor   m_main_cursor;
+    
     Button      *m_button_stop;
     Button      *m_button_play;
+    Button      *m_button_test;
 
     Button      *m_button_perfedit;
 
@@ -95,51 +109,46 @@ class mainwnd : public Gtk::Window
 
     Entry       *m_entry_notes;
 
-    Connection   m_timeout_connect;
+    SigC::Connection   m_timeout_connect;
 
-    int      m_screen_set;
-
-	void new_file( void );
- 
+    void file_new_dialog( void );
     void file_save_dialog( void );
+    void file_saveas_dialog( void );
     void file_open_dialog( void );
+    void file_import_dialog( void );
+    void file_exit_dialog( void );
 
     void options_dialog( void );
     void about_dialog( void );
 
-    void file_save( bool );
-    void file_open( bool );
 
     void adj_callback_ss( );
     void adj_callback_bpm( );
 
     void edit_callback_notepad( );
 
-    int timer_callback();
+    bool timer_callback( );
 
     void start_playing();
     void stop_playing();
+    void test();
 
     void open_performance_edit( );
 
     void sequence_key( int a_seq );
 
-    map<long,long> key_events;
-    //bool m_l_control_pressed;
-    //bool m_r_control_pressed;
-    //bool m_l_alt_pressed;
-
-    //bool m_sequence_state[c_total_seqs];
+    void set_window_title_filename( std::string a_file );
 
  public:
 
     mainwnd(perform *a_p);
+    ~mainwnd();
+
+    bool on_delete_event(GdkEventAny *a_e);
+    bool on_key_press_event(GdkEventKey* a_ev);
+    bool on_key_release_event(GdkEventKey* a_ev);
 
-    void destroy_func();
 
-    int delete_event_impl(GdkEventAny *a_e);
-    int key_press_event_impl(GdkEventKey* a_ev);
-    int key_release_event_impl(GdkEventKey* a_ev);
 };
 
 
