@@ -71,7 +71,7 @@ midibus::unlock( )
 
 
 
-void midibus::init_out( )
+bool midibus::init_out( )
 {
     /* temp return */
     int ret;
@@ -86,8 +86,8 @@ void midibus::init_out( )
     m_local_addr_port = ret;
 
     if ( ret < 0 ){
-	printf( "snd_seq_create_simple_port(write) error\n");
-
+        printf( "snd_seq_create_simple_port(write) error\n");
+        return false;
     }
 
     /* connect to */
@@ -96,13 +96,16 @@ void midibus::init_out( )
 			      m_dest_addr_client, 
 			      m_dest_addr_port );
     if ( ret < 0 ){
-	printf( "snd_seq_connect_to(%d:%d) error\n", m_dest_addr_client, m_dest_addr_port);
+        printf( "snd_seq_connect_to(%d:%d) error\n", m_dest_addr_client, m_dest_addr_port);
+        return false;
     }
+
+    return true;
 }
 
 
 
-void midibus::init_in( )
+bool midibus::init_in( )
 {
 
     /* temp return */
@@ -110,15 +113,16 @@ void midibus::init_in( )
 
     /* create ports */
     ret = snd_seq_create_simple_port(m_seq, 
-				     "seq24 in",
-     				     SND_SEQ_PORT_CAP_NO_EXPORT |
-				     SND_SEQ_PORT_CAP_WRITE,
-				     SND_SEQ_PORT_TYPE_MIDI_GENERIC |
-				     SND_SEQ_PORT_TYPE_APPLICATION );
+                                     "seq24 in",
+                                     SND_SEQ_PORT_CAP_NO_EXPORT |
+                                     SND_SEQ_PORT_CAP_WRITE,
+                                     SND_SEQ_PORT_TYPE_MIDI_GENERIC |
+                                     SND_SEQ_PORT_TYPE_APPLICATION );
     m_local_addr_port = ret;
 
     if ( ret < 0 ){
-	printf( "snd_seq_create_simple_port(read) error\n");
+        printf( "snd_seq_create_simple_port(read) error\n");
+        return false;
     }
 
     snd_seq_port_subscribe_t *subs;
@@ -143,9 +147,11 @@ void midibus::init_in( )
     ret = snd_seq_subscribe_port(m_seq, subs);
 
     if ( ret < 0 ){
-	printf( "snd_seq_connect_from(%d:%d) error\n", m_dest_addr_client, m_dest_addr_port);
-	exit(1);
+        printf( "snd_seq_connect_from(%d:%d) error\n", m_dest_addr_client, m_dest_addr_port);
+        return false;
     }
+
+    return true;
 }
 
 
@@ -473,16 +479,22 @@ mastermidibus::unlock( )
 void 
 mastermidibus::start()
 {
+    lock();
+         
     /* start timer */
     snd_seq_start_queue( m_alsa_seq, m_queue, NULL );
     
     for ( int i=0; i < m_num_out_buses; i++ )
 	m_buses_out[i]->start();
+
+     unlock();
 }
 
 void 
 mastermidibus::stop()
 {
+    lock();
+         
     snd_seq_drain_output( m_alsa_seq );
     snd_seq_sync_output_queue( m_alsa_seq );
 
@@ -491,6 +503,8 @@ mastermidibus::stop()
 
     for ( int i=0; i < m_num_out_buses; i++ )
 	m_buses_out[i]->stop();
+
+     unlock();
 }
 
 
@@ -498,8 +512,12 @@ mastermidibus::stop()
 void
 mastermidibus::clock( long a_tick )
 {
+    lock();
+    
     for ( int i=0; i < m_num_out_buses; i++ )
 	m_buses_out[i]->clock( a_tick );
+    
+    unlock();
 }
 
 void 
@@ -636,9 +654,12 @@ mastermidibus::mastermidibus()
 				     snd_seq_port_info_get_name(pinfo),
 				     m_num_out_buses, m_queue );
 		    
-		    m_buses_out[m_num_out_buses]->init_out();
-		    m_buses_out_active[m_num_out_buses] = true;		
-			m_buses_out_init[m_num_out_buses] = true;		
+		    if ( m_buses_out[m_num_out_buses]->init_out() ){
+                m_buses_out_active[m_num_out_buses] = true;		
+                m_buses_out_init[m_num_out_buses] = true;
+            } else {
+                m_buses_out_init[m_num_out_buses] = true;	
+            }
 
 		    m_num_out_buses++;
 		}	
@@ -656,9 +677,12 @@ mastermidibus::mastermidibus()
 				     snd_seq_port_info_get_name(pinfo),
 				      m_num_in_buses, m_queue);
 		    	
-			m_buses_in[m_num_in_buses]->init_in();
-			m_buses_in_active[m_num_in_buses] = true;	
-			m_buses_in_init[m_num_in_buses] = true;	
+			if ( m_buses_in[m_num_in_buses]->init_in() ){
+                m_buses_in_active[m_num_in_buses] = true;	
+                m_buses_in_init[m_num_in_buses] = true;
+            } else {
+                m_buses_in_init[m_num_in_buses] = true;	
+            }
 		    m_num_in_buses++;
 		}	
 	    }
@@ -752,9 +776,11 @@ mastermidibus::play( unsigned char a_bus, event *a_e24, unsigned char a_channel
 void 
 mastermidibus::set_clock( unsigned char a_bus, bool a_clocking )
 {
-	if ( m_buses_out_active[a_bus] && a_bus < m_num_out_buses ){
-		m_buses_out[a_bus]->set_clock( a_clocking );
-	}
+    lock();
+    if ( m_buses_out_active[a_bus] && a_bus < m_num_out_buses ){
+        m_buses_out[a_bus]->set_clock( a_clocking );
+    }
+    unlock();
 }
 
 bool 
