@@ -89,8 +89,8 @@ class midibus
 
     ~midibus();
 
-    void init_out(  );
-    void init_in(  );
+    bool init_out(  );
+    bool init_in(  );
 
     void print();
 
