@@ -82,7 +82,7 @@ midifile::read_var()
 }
 
 
-void 
+bool
 midifile::parse( perform *a_perf, int a_screen_set )
 {
 
@@ -91,7 +91,10 @@ midifile::parse( perform *a_perf, int a_screen_set )
 
     /* open binary file */
     ifstream file ( m_name.c_str(), ios::in | ios::binary | ios::ate );
-  
+
+    if( ! file.is_open() )
+        return false;
+    
     file_size = file.tellg();
 
     /* run to start */
@@ -145,11 +148,11 @@ midifile::parse( perform *a_perf, int a_screen_set )
 
     /* magic number 'MThd' */
     if ( ID != 0x4D546864 ) 
-	throw;
+	return false;
 
     /* we are only supporting format 1 for now */
     if ( Format != 1 )
-	throw;
+	return false;
 
     /* We should be good to load now   */
     /* for each Track in the midi file */
@@ -369,7 +372,7 @@ midifile::parse( perform *a_perf, int a_screen_set )
 			
 			/* sysex */	 
 			//printf( "no SYSEX code in perform.C yet\n");
-			throw;
+			return false;
 		    }
 
 		    break;
@@ -377,7 +380,7 @@ midifile::parse( perform *a_perf, int a_screen_set )
 		default:
 		    
 		    //printf( "switch(status) error\n");
-		    throw;
+		    return false;
 		    break;
 		}
 
@@ -495,6 +498,7 @@ midifile::parse( perform *a_perf, int a_screen_set )
 
 
     delete[] m_d;
+    return true;
     //printf ( "done\n");
 }
 
@@ -517,13 +521,16 @@ midifile::write_short( unsigned short a_x )
     m_l.push_front(  (a_x & 0x00FF)       );
 }
 
-void 
+bool 
 midifile::write( perform *a_perf  )
 {
     /* open binary file */
  
     ofstream file ( m_name.c_str(), ios::out | ios::binary | ios::trunc  );
 
+    if( ! file.is_open() )
+        return false;
+    
      /* used in small loops */
     int i;
 
@@ -580,39 +587,13 @@ midifile::write( perform *a_perf  )
 	
     /* midi control */
     write_long( c_midictrl );
-    write_long( c_seqs_in_set );
-    for (i=0; i< c_seqs_in_set; i++ ){
-		
-		m_l.push_front( a_perf->get_midi_control_toggle(i)->m_active );
-		m_l.push_front( a_perf->get_midi_control_toggle(i)->m_inverse_active );
-		m_l.push_front( a_perf->get_midi_control_toggle(i)->m_status );
-		m_l.push_front( a_perf->get_midi_control_toggle(i)->m_data );
-		m_l.push_front( a_perf->get_midi_control_toggle(i)->m_min_value );
-		m_l.push_front( a_perf->get_midi_control_toggle(i)->m_max_value );
-
-		m_l.push_front( a_perf->get_midi_control_on(i)->m_active );
-		m_l.push_front( a_perf->get_midi_control_on(i)->m_inverse_active );
-		m_l.push_front( a_perf->get_midi_control_on(i)->m_status );
-		m_l.push_front( a_perf->get_midi_control_on(i)->m_data );
-		m_l.push_front( a_perf->get_midi_control_on(i)->m_min_value );
-		m_l.push_front( a_perf->get_midi_control_on(i)->m_max_value );
-
-		m_l.push_front( a_perf->get_midi_control_off(i)->m_active );
-		m_l.push_front( a_perf->get_midi_control_off(i)->m_inverse_active );
-		m_l.push_front( a_perf->get_midi_control_off(i)->m_status );
-		m_l.push_front( a_perf->get_midi_control_off(i)->m_data );
-		m_l.push_front( a_perf->get_midi_control_off(i)->m_min_value );
-		m_l.push_front( a_perf->get_midi_control_off(i)->m_max_value );
-    }
-
+    write_long( 0 );
+ 
 
     /* bus mute/unmute data */
-    int buses = a_perf->get_master_midi_bus( )->get_num_out_buses();
     write_long( c_midiclocks );
-    write_long( buses );
-    for (i=0; i< buses; i++ ){
-		m_l.push_front( (char) a_perf->get_master_midi_bus( )->get_clock(i));
-    }
+    write_long( 0 );
+  
 
     /* notepad data */
     write_long( c_notes );
@@ -659,6 +640,7 @@ midifile::write( perform *a_perf  )
 
     delete[] m_d;
 
+    return true;
 
 
 }
