@@ -18,7 +18,6 @@
 //
 //-----------------------------------------------------------------------------
 
-
 #ifndef SEQ24_SEQEDIT
 #define SEQ24_SEQEDIT
 
@@ -30,31 +29,33 @@
 #include "seqevent.h"
 #include "perform.h"
 
-#include <gtk--/adjustment.h>
-#include <gtk--/button.h>
-#include <gtk--/window.h>
-#include <gtk--/accelgroup.h>
-#include <gtk--/box.h>
-#include <gtk--/main.h>
-#include <gtk--/menu.h>
-#include <gtk--/menubar.h>
-#include <gtk--/eventbox.h>
-#include <gtk--/window.h>
-#include <gtk--/table.h>
-#include <gtk--/drawingarea.h>
-#include <gtk--/widget.h>
-#include <gtk--/scrollbar.h>
-#include <gtk--/viewport.h>
-#include <gtk--/combo.h>
-#include <gtk--/label.h>
-#include <gtk--/toolbar.h>
-#include <gtk--/optionmenu.h> 
-#include <gtk--/togglebutton.h>
-#include <gtk--/invisible.h>
-#include <gtk--/separator.h> 
-#include <gtk--/pixmap.h> 
-#include <gtk--/tooltips.h>
-#include <gtk--/invisible.h>
+#include <gtkmm/adjustment.h>
+#include <gtkmm/button.h>
+#include <gtkmm/window.h>
+#include <gtkmm/accelgroup.h>
+#include <gtkmm/box.h>
+#include <gtkmm/main.h>
+#include <gtkmm/menu.h>
+#include <gtkmm/menubar.h>
+#include <gtkmm/eventbox.h>
+#include <gtkmm/window.h>
+#include <gtkmm/table.h>
+#include <gtkmm/drawingarea.h>
+#include <gtkmm/widget.h>
+#include <gtkmm/scrollbar.h>
+#include <gtkmm/viewport.h>
+#include <gtkmm/combo.h>
+#include <gtkmm/label.h>
+#include <gtkmm/toolbar.h>
+#include <gtkmm/optionmenu.h> 
+#include <gtkmm/togglebutton.h>
+#include <gtkmm/invisible.h>
+#include <gtkmm/separator.h> 
+#include <gtkmm/image.h>
+#include <gtkmm/tooltips.h>
+#include <gtkmm/invisible.h>
+#include <gtkmm/image.h>
+
 
 #include <list>
 #include <string>
@@ -79,7 +80,7 @@ class seqedit : public Gtk::Window
     Menu       *m_menu_tools;
     Menu       *m_menu_zoom;
     Menu       *m_menu_snap;
-	Menu       *m_menu_note_length;
+    Menu       *m_menu_note_length;
 
     /* length in measures */
     Menu       *m_menu_length;
@@ -88,6 +89,7 @@ class seqedit : public Gtk::Window
     Menu       *m_menu_data;
     Menu       *m_menu_key;
     Menu       *m_menu_scale;
+    Menu       *m_menu_sequences;
 
     /* time signature, beats per measure, beat width */
     Menu       *m_menu_bpm;
@@ -111,39 +113,51 @@ class seqedit : public Gtk::Window
     VBox       *m_vbox;
     HBox       *m_hbox;
     HBox       *m_hbox2;
+    HBox       *m_hbox3;
 
-    Viewport   *m_keysview;
-    Viewport   *m_rollview;
-    Viewport   *m_dataview;
-    Viewport   *m_timeview;
-    Viewport   *m_eventview;
+    //Viewport   *m_keysview;
+    //Viewport   *m_rollview;
+    //Viewport   *m_dataview;
+    //Viewport   *m_timeview;
+    //Viewport   *m_eventview;
 
-    VScrollbar *m_vscroll;
-    HScrollbar *m_hscroll;
+    //VScrollbar *m_vscroll;
+    //HScrollbar *m_hscroll;
 
-	Button      *m_button_tools;
+    Adjustment *m_vadjust;
+    Adjustment *m_hadjust;
 
+    VScrollbar *m_vscroll_new;
+    HScrollbar *m_hscroll_new;
+    
+    Button      *m_button_undo;
+    
+    Button      *m_button_tools;
+
+    Button      *m_button_sequence;
+    Entry       *m_entry_sequence;
+    
     Button      *m_button_bus;
     Entry       *m_entry_bus;
-
+    
     Button      *m_button_channel;
     Entry       *m_entry_channel;
- 
+    
     Button      *m_button_snap;
     Entry       *m_entry_snap;
-
-	Button      *m_button_note_length;
+    
+    Button      *m_button_note_length;
     Entry       *m_entry_note_length;
-
+    
     Button      *m_button_zoom;
     Entry       *m_entry_zoom;
-
+    
     Button      *m_button_length;
     Entry       *m_entry_length;
-
+    
     Button      *m_button_key;
     Entry       *m_entry_key;
-
+    
     Button      *m_button_scale;
     Entry       *m_entry_scale;
 
@@ -173,8 +187,8 @@ class seqedit : public Gtk::Window
     int         m_snap;
     static int  m_initial_snap;
 
-	int         m_note_length;
-	static int  m_initial_note_length;
+    int         m_note_length;
+    static int  m_initial_note_length;
 
     /* music scale and key */
     int         m_scale;
@@ -183,6 +197,9 @@ class seqedit : public Gtk::Window
     int         m_key;
     static int  m_initial_key;
 
+    int         m_sequence;
+    static int  m_initial_sequence;
+
     long        m_measures;
 
     /* what is the data window currently editing ? */
@@ -191,7 +208,7 @@ class seqedit : public Gtk::Window
 
     void set_zoom( int a_zoom );
     void set_snap( int a_snap );
-	void set_note_length( int a_note_length );
+    void set_note_length( int a_note_length );
 
     void set_bpm( int a_beats_per_measure );
     void set_bw( int a_beat_width );
@@ -204,11 +221,14 @@ class seqedit : public Gtk::Window
 
     void set_scale( int a_scale );
     void set_key( int a_note );
+
+    void set_background_sequence( int a_seq );
     
     void name_change_callback( void );
     void play_change_callback( void );
     void record_change_callback( void );
     void thru_change_callback( void );
+    void undo_callback( void );
 
     void set_data_type( unsigned char a_status, 
 			unsigned char a_control = 0 );
@@ -221,12 +241,18 @@ class seqedit : public Gtk::Window
     void menu_action_quantise( void );
     
     void popup_menu( Menu *a_menu );
+    void popup_event_menu( void );                                                                                                                                                                                                                                                                                                
+    void popup_midibus_menu( void );
+    void popup_sequence_menu( void );
+    void popup_tool_menu( void );
 
-    void realize_impl();
+    Gtk::Image* create_menu_image( bool a_state = false );
+    
+    void on_realize();
 
-    int timeout( void );
+    bool timeout( void );
 
-	void do_action( int a_action );
+    void do_action( int a_action, int a_var );
 
  public:
 
@@ -239,11 +265,8 @@ class seqedit : public Gtk::Window
 
  
 
-    int delete_event_impl(GdkEventAny *a_event);
-
-    int focus_in_event_impl( GdkEventFocus* a_ev );
-    int focus_out_event_impl( GdkEventFocus* a_ev );
- 
+    bool on_delete_event(GdkEventAny *a_event);
+    
 };
 
 
