@@ -25,18 +25,19 @@
 #include "sequence.h"
 #include "seqkeys.h"
 
-#include <gtk--/button.h>
-#include <gtk--/window.h>
-#include <gtk--/accelgroup.h>
-#include <gtk--/box.h>
-#include <gtk--/main.h>
-#include <gtk--/menu.h>
-#include <gtk--/menubar.h>
-#include <gtk--/eventbox.h>
-#include <gtk--/window.h>
-#include <gtk--/table.h>
-#include <gtk--/drawingarea.h>
-#include <gtk--/widget.h>
+#include <gtkmm/button.h>
+#include <gtkmm/window.h>
+#include <gtkmm/accelgroup.h>
+#include <gtkmm/box.h>
+#include <gtkmm/main.h>
+#include <gtkmm/menu.h>
+#include <gtkmm/menubar.h>
+#include <gtkmm/eventbox.h>
+#include <gtkmm/window.h>
+#include <gtkmm/table.h>
+#include <gtkmm/drawingarea.h>
+#include <gtkmm/widget.h>
+#include <gtkmm/adjustment.h>
 
 #include "globals.h"
 #include "seqdata.h"
@@ -60,18 +61,18 @@ class seqroll : public Gtk::DrawingArea
 
  private: 
 
-    Gdk_GC       m_gc;
-    Gdk_Window   m_window;
-    Gdk_Color    m_black, m_white, m_grey, m_dk_grey;
+    Glib::RefPtr<Gdk::GC> m_gc;
+    Glib::RefPtr<Gdk::Window>   m_window;
+    Gdk::Color    m_black, m_white, m_grey, m_dk_grey, m_red;
 
-    Gdk_Pixmap   m_pixmap;
-    Gdk_Pixmap   m_background;
+    Glib::RefPtr<Gdk::Pixmap> m_pixmap;
  
     rect         m_old;
     rect         m_selected;
 
     sequence     *m_seq;
     sequence     *m_clipboard;
+    perform      *m_perform;
     seqdata      *m_seqdata_wid;
     seqevent     *m_seqevent_wid;
     seqkeys      *m_seqkeys_wid;
@@ -97,7 +98,7 @@ class seqroll : public Gtk::DrawingArea
     /* when highlighting a bunch of events */
     bool m_selecting;
     bool m_moving;
-	bool m_moving_init;
+    bool m_moving_init;
     bool m_growing;
     bool m_adding;
     bool m_paste;
@@ -105,22 +106,39 @@ class seqroll : public Gtk::DrawingArea
     /* where the dragging started */
     int m_drop_x; 
     int m_drop_y;
+    int m_move_delta_x;
+    int m_move_delta_y;
     int m_current_x;
     int m_current_y;
 
+    int m_move_snap_offset_x;
+
     int m_old_progress_x;
 
-    void realize_impl();
-    int expose_event_impl(GdkEventExpose* a_ev);
-    int button_press_event_impl(GdkEventButton* a_ev); 
-    int button_release_event_impl(GdkEventButton* a_ev);
-    int motion_notify_event_impl(GdkEventMotion* a_ev);
-    int key_press_event_impl(GdkEventKey* a_p0);
-    int focus_in_event_impl(GdkEventFocus*);
-    int focus_out_event_impl(GdkEventFocus*);
+    Adjustment   *m_vadjust;
+    Adjustment   *m_hadjust;
 
-    int leave_notify_event_impl	(GdkEventCrossing* a_p0);
-    int enter_notify_event_impl	(GdkEventCrossing* a_p0);
+    int m_scroll_offset_ticks;
+    int m_scroll_offset_key;
+
+    int m_scroll_offset_x;
+    int m_scroll_offset_y;
+
+    int m_background_sequence;
+    bool m_drawing_background_seq;
+
+    
+    void on_realize();
+    bool on_expose_event(GdkEventExpose* a_ev);
+    bool on_button_press_event(GdkEventButton* a_ev); 
+    bool on_button_release_event(GdkEventButton* a_ev);
+    bool on_motion_notify_event(GdkEventMotion* a_ev);
+    bool on_key_press_event(GdkEventKey* a_p0);
+    bool on_focus_in_event(GdkEventFocus*);
+    bool on_focus_out_event(GdkEventFocus*);
+
+    bool on_leave_notify_event	(GdkEventCrossing* a_p0);
+    bool on_enter_notify_event	(GdkEventCrossing* a_p0);
 
 
     void convert_xy( int a_x, int a_y, long *a_ticks, int *a_note);
@@ -139,14 +157,24 @@ class seqroll : public Gtk::DrawingArea
 				 int *a_x, int *a_y, 
 				 int *a_w, int *a_h );
 	
-    void draw_events_on( Gdk_Drawable *a_draw );
+    void draw_events_on(  Glib::RefPtr<Gdk::Drawable> a_draw );
 
   
     int idle_progress();
+    
+    void on_size_allocate(GtkAllocation* );
+
+    void change_horz( void );
+    void change_vert( void );
+
+    void force_draw( void );
+
+
 
  public:
 
     void reset();
+    void redraw();
     void set_zoom( int a_zoom );
     void set_snap( int a_snap );
 	void set_note_length( int a_note_length );
@@ -155,9 +183,8 @@ class seqroll : public Gtk::DrawingArea
     void set_key( int a_key );
     
     void update_sizes();
-    void fill_background_pixmap();
+    void draw_background();
     void draw_events_on_pixmap();
-    void draw_pixmap_on_window();
     void draw_selection_on_window();
     void update_pixmap();
     int idle_redraw();
@@ -166,13 +193,18 @@ class seqroll : public Gtk::DrawingArea
     void set_adding( bool a_adding );
 
     void start_paste( );
+    
+    void set_background_sequence( bool a_state, int a_seq );
 
-    seqroll( sequence *a_seq, int a_zoom, int a_snap, 
-	     seqdata *a_seqdata_wid, 
-	     seqevent *a_seqevent_wid,
-	     seqkeys *a_seqkeys_wid,
-	     mainwid *a_mainwid, 
-	     int a_pos );
+    seqroll( perform *a_perf,
+             sequence *a_seq, int a_zoom, int a_snap, 
+             seqdata *a_seqdata_wid, 
+             seqevent *a_seqevent_wid,
+             seqkeys *a_seqkeys_wid,
+             mainwid *a_mainwid, 
+             int a_pos, 
+             Adjustment *a_hadjust,
+             Adjustment *a_vadjust );
 
 	void set_data_type( unsigned char a_status, unsigned char a_control  );
  
