@@ -23,10 +23,7 @@
 
 sequence::sequence( )
 {
-    m_list_event     = new list<event>();
-    m_list_clipboard = new list<event>();
-    m_list_trigger   = new list<trigger>();
-
+ 
     m_editing       = false;
     m_playing       = false;
     m_was_playing   = false;
@@ -50,9 +47,35 @@ sequence::sequence( )
 
     m_last_tick = 0;
 
-	m_masterbus = NULL;
-	m_dirty_main = true;
-	m_dirty_edit = true;
+    m_masterbus = NULL;
+    m_dirty_main = false;
+    m_dirty_edit = false;
+}
+
+void 
+sequence::push_undo( void )
+{
+	lock();
+
+	m_list_undo.push( m_list_event ); 
+
+	unlock();
+}
+
+
+void 
+sequence::pop_undo( void )
+{
+	lock();
+
+	if (m_list_undo.size() > 0 ){
+
+		m_list_event = m_list_undo.top();
+		m_list_undo.pop();
+		verify_and_link();
+	}
+
+	unlock();
 }
 
 
@@ -88,7 +111,7 @@ sequence::set_bw( long a_beat_width )
 {
 	lock();
 	m_time_beat_width = a_beat_width;
-	m_dirty_main = m_dirty_edit = true;
+	m_dirty_main = true;
 	unlock();
 }
 
@@ -101,8 +124,7 @@ sequence::get_bw( void )
 
 sequence::~sequence()
 {
-    delete m_list_event;
-    delete m_list_clipboard;
+ 
 }
 
 /* adds event in sorted manner */
@@ -111,8 +133,8 @@ sequence::add_event( const event *a_e )
 {
     lock();
 
-    m_list_event->push_front( *a_e );
-    m_list_event->sort( );
+    m_list_event.push_front( *a_e );
+    m_list_event.sort( );
 
     reset_draw_marker();
 
@@ -133,7 +155,7 @@ sequence::toggle_queued( void )
 {
 	lock();
 
-	m_dirty_main = m_dirty_edit = true;
+	m_dirty_main = true;
 
 	m_queued = !m_queued;
 	m_queued_tick = m_last_tick - (m_last_tick % m_length) + m_length;
@@ -147,7 +169,7 @@ sequence::off_queued( void )
 
 	lock();
 
-	m_dirty_main = m_dirty_edit = true;
+	m_dirty_main = true;
 
 	m_queued = false;
 
@@ -190,15 +212,14 @@ sequence::play( long a_tick, bool a_playback_mode )
 
   
 	
-  	bool trigger_state = m_playing;
+  	bool trigger_state = false;
 	long trigger_tick = 0;
-	bool triggers = false;
 
-	list<trigger>::iterator i = m_list_trigger->begin();
+	list<trigger>::iterator i = m_list_trigger.begin();
 
 	//printf( "start[%ld] => <= end[%ld]\n", start_tick, end_tick );
 
-   	while ( i != m_list_trigger->end()){
+   	while ( i != m_list_trigger.end()){
 	 
 	    //printf( "    tick[%ld]\n", (*i).m_tick );
    
@@ -206,9 +227,6 @@ sequence::play( long a_tick, bool a_playback_mode )
 		
   		trigger_state = (*i).m_state;
   		trigger_tick = (*i).m_tick;
-		
-		/* we have a change */
-		triggers = true;
 	    } 
 	    else if ( (*i).m_tick >  end_tick ){
 		
@@ -219,8 +237,7 @@ sequence::play( long a_tick, bool a_playback_mode )
 	}
 
 	/* we had triggers in our slice and its not equal to current state */
-	if ( triggers &&
-	     trigger_state != m_playing ){
+	if ( trigger_state != m_playing ){
 
 	    //printf( "trigger %d\n", trigger_state );
 
@@ -243,7 +260,7 @@ sequence::play( long a_tick, bool a_playback_mode )
 
 	}
 
-	if( m_list_trigger->size() == 0 && 
+	if( m_list_trigger.size() == 0 && 
 	    m_playing ){
 	    
 	    set_playing(false);
@@ -254,9 +271,9 @@ sequence::play( long a_tick, bool a_playback_mode )
     /* play the notes in our frame */
     if ( m_playing ){
 	
-	list<event>::iterator e = m_list_event->begin();
+	list<event>::iterator e = m_list_event.begin();
 	
-	while ( e != m_list_event->end()){
+	while ( e != m_list_event.end()){
 
 	    //printf ( "s[%ld] -> t[%ld] ", start_tick, end_tick  ); (*e).print();
  	    if ( ((*e).get_timestamp() + offset_base ) >= start_tick  &&
@@ -276,9 +293,9 @@ sequence::play( long a_tick, bool a_playback_mode )
 	    e++;
 	    
 	    /* did we hit the end ? */
-	    if ( e == m_list_event->end() ){
+	    if ( e == m_list_event.end() ){
 		
-		e = m_list_event->begin();
+		e = m_list_event.begin();
 		offset_base += m_length;
 	    }
 	}
@@ -334,15 +351,15 @@ sequence::play( long a_tick, bool a_playback_mode )
 //     /* if we are using our in sequence on/off triggers */
 //     if ( a_playback_mode ){
 
-//  	list<trigger>::iterator i = m_list_trigger->begin();
+//  	list<trigger>::iterator i = m_list_trigger.begin();
 	
 //  	bool trigger_state = m_playing;
 //  	long trigger_tick = 0;
 
-//  	if ( m_list_trigger->size() == 0 )
+//  	if ( m_list_trigger.size() == 0 )
 //  	    trigger_state = false;
 
-//  	while ( i != m_list_trigger->end()){
+//  	while ( i != m_list_trigger.end()){
 
 //  	    if ( (*i).m_tick < a_tick ||
 // 		 (*i).m_tick == start_tick ){
@@ -397,9 +414,9 @@ sequence::play( long a_tick, bool a_playback_mode )
 //     /* play the notes in our frame */
 //     if ( m_playing ){
 	
-// 	list<event>::iterator e = m_list_event->begin();
+// 	list<event>::iterator e = m_list_event.begin();
 	
-// 	while ( e != m_list_event->end()){
+// 	while ( e != m_list_event.end()){
 
 // 	    //printf ( "s[%ld] -> t[%ld] ", start_tick, a_tick  ); (*e).print();
 //  	    if ( ((*e).get_timestamp() + offset_base) >= start_tick  &&
@@ -426,9 +443,9 @@ sequence::play( long a_tick, bool a_playback_mode )
 // 	    e++;
 	    
 // 	    /* did we hit the end ? */
-// 	    if ( e == m_list_event->end() ){
+// 	    if ( e == m_list_event.end() ){
 		
-// 		e = m_list_event->begin();
+// 		e = m_list_event.begin();
 // 		offset_base += m_length;
 // 	    }
 // 	}
@@ -473,15 +490,15 @@ sequence::verify_and_link()
     lock();
 
     /* unselect all */
-    for ( i = m_list_event->begin(); i != m_list_event->end(); i++ ){
+    for ( i = m_list_event.begin(); i != m_list_event.end(); i++ ){
 	(*i).unselect();
 	(*i).clear_link();
     }
 
-    on = m_list_event->begin();
+    on = m_list_event.begin();
 	    
     /* pair ons and offs */
-    while ( on != m_list_event->end() ){
+    while ( on != m_list_event.end() ){
 
 	/* check for a note on, then look for its
 	   note off */
@@ -490,7 +507,7 @@ sequence::verify_and_link()
 	    /* get next possible off node */
 	    off = on; off++;
 
-	    while ( off != m_list_event->end() ){
+	    while ( off != m_list_event.end() ){
 		
 		/* is a off event, == notes, and isnt
 		   selected  */
@@ -513,12 +530,12 @@ sequence::verify_and_link()
     }
 
     /* unselect all */
-    for ( i = m_list_event->begin(); i != m_list_event->end(); i++ ){
+    for ( i = m_list_event.begin(); i != m_list_event.end(); i++ ){
 	(*i).unselect();
     }
 
     /* kill those not in range */
-    for ( i = m_list_event->begin(); i != m_list_event->end(); i++ ){
+    for ( i = m_list_event.begin(); i != m_list_event.end(); i++ ){
 	
 	/* if our current time stamp is greater then the length */
 	if ( (*i).get_timestamp() >= m_length ||
@@ -547,10 +564,10 @@ sequence::link_new( )
 
     lock();
 
-    on = m_list_event->begin();
+    on = m_list_event.begin();
 
     /* pair ons and offs */
-    while ( on != m_list_event->end()){
+    while ( on != m_list_event.end()){
 
 	/* check for a note on, then look for its
 	   note off */
@@ -560,7 +577,7 @@ sequence::link_new( )
 	    /* get next element */
 	    off = on; off++;
 	    
-	    while ( off != m_list_event->end()){
+	    while ( off != m_list_event.end()){
 
 		/* is a off event, == notes, and isnt
 		   selected  */
@@ -592,8 +609,8 @@ sequence::remove_selected( )
 
     lock();
 
-    i = m_list_event->begin();
-    while( i != m_list_event->end() ){
+    i = m_list_event.begin();
+    while( i != m_list_event.end() ){
 	
 	if ((*i).is_selected()){
 	    
@@ -607,7 +624,7 @@ sequence::remove_selected( )
 	    }
 	   
 	    t = i; t++;
-	    m_list_event->erase(i);
+	    m_list_event.erase(i);
 	    i = t;
 	}
 	else {
@@ -640,7 +657,7 @@ sequence::get_selected_box( long *a_tick_s, int *a_note_h,
 
     lock();
 
-    for ( i = m_list_event->begin(); i != m_list_event->end(); i++ ){
+    for ( i = m_list_event.begin(); i != m_list_event.end(); i++ ){
 
 	if( (*i).is_selected() ){
 	    
@@ -677,11 +694,11 @@ sequence::get_clipboard_box( long *a_tick_s, int *a_note_h,
     
     lock();
 
-    if ( m_list_clipboard->size() == 0 ) {
+    if ( m_list_clipboard.size() == 0 ) {
 	*a_tick_s = *a_tick_f = *a_note_h = *a_note_l = 0; 
     }
 
-    for ( i = m_list_clipboard->begin(); i != m_list_clipboard->end(); i++ ){
+    for ( i = m_list_clipboard.begin(); i != m_list_clipboard.end(); i++ ){
 	
 	time = (*i).get_timestamp();
 	
@@ -708,7 +725,7 @@ sequence::get_num_selected_notes( )
 
     lock();
 
-    for ( i = m_list_event->begin(); i != m_list_event->end(); i++ ){
+    for ( i = m_list_event.begin(); i != m_list_event.end(); i++ ){
 
         if( (*i).is_note_on()                &&
 			(*i).is_selected() ){
@@ -732,7 +749,7 @@ sequence::get_num_selected_events( unsigned char a_status,
 
 	lock();
 
-    for ( i = m_list_event->begin(); i != m_list_event->end(); i++ ){
+    for ( i = m_list_event.begin(); i != m_list_event.end(); i++ ){
 
 		if( (*i).get_status()    == a_status ){
 
@@ -765,13 +782,18 @@ sequence::select_note_events( long a_tick_s, int a_note_h,
 
     lock();
 
-    for ( i = m_list_event->begin(); i != m_list_event->end(); i++ ){
+    
+    for ( i = m_list_event.begin(); i != m_list_event.end(); i++ ){
 
+        
+
+        
         if( (*i).is_note_off()                &&
             (*i).get_timestamp() >= a_tick_s &&
             (*i).get_note()      <= a_note_h &&
             (*i).get_note()      >= a_note_l ){
 
+              
 	    if ( (*i).is_linked() ){
 
 		event *ev = (*i).get_linked();
@@ -818,7 +840,7 @@ sequence::select_events( long a_tick_s, long a_tick_f,
 
     lock();
 
-    for ( i = m_list_event->begin(); i != m_list_event->end(); i++ ){
+    for ( i = m_list_event.begin(); i != m_list_event.end(); i++ ){
 
 	if( (*i).get_status()    == a_status && 
 	    (*i).get_timestamp() >= a_tick_s &&
@@ -852,7 +874,7 @@ sequence::select_all( void )
 
     list<event>::iterator i;
 
-    for ( i = m_list_event->begin(); i != m_list_event->end(); i++ )
+    for ( i = m_list_event.begin(); i != m_list_event.end(); i++ )
 	(*i).select( );
 
     unlock();
@@ -866,7 +888,7 @@ sequence::unselect( void )
     lock();
 
     list<event>::iterator i;
-    for ( i = m_list_event->begin(); i != m_list_event->end(); i++ )
+    for ( i = m_list_event.begin(); i != m_list_event.end(); i++ )
 	(*i).unselect();
 
     unlock();
@@ -883,7 +905,7 @@ sequence::move_selected_notes( long a_delta_tick, int a_delta_note )
 
     list<event>::iterator i;
 
-    for ( i = m_list_event->begin(); i != m_list_event->end(); i++ ){
+    for ( i = m_list_event.begin(); i != m_list_event.end(); i++ ){
 	
 	/* is it being moved ? */
 	if ( (*i).is_selected() ){
@@ -910,6 +932,7 @@ sequence::move_selected_notes( long a_delta_tick, int a_delta_note )
     unlock();
 }
 
+    
 
 
 
@@ -923,7 +946,7 @@ sequence::grow_selected( long a_delta_tick )
 
     list<event>::iterator i;
 
-    for ( i = m_list_event->begin(); i != m_list_event->end(); i++ ){
+    for ( i = m_list_event.begin(); i != m_list_event.end(); i++ ){
 	
 	if ( (*i).is_selected() &&
 	     (*i).is_note_on() &&
@@ -966,7 +989,7 @@ sequence::increment_selected( unsigned char a_status, unsigned char a_control )
 
     list<event>::iterator i;
 
-    for ( i = m_list_event->begin(); i != m_list_event->end(); i++ ){
+    for ( i = m_list_event.begin(); i != m_list_event.end(); i++ ){
 	
 		if ( (*i).is_selected() &&
 			 (*i).get_status() == a_status ){
@@ -999,7 +1022,7 @@ sequence::decrement_selected(unsigned char a_status, unsigned char a_control )
 
     list<event>::iterator i;
 
-    for ( i = m_list_event->begin(); i != m_list_event->end(); i++ ){
+    for ( i = m_list_event.begin(); i != m_list_event.end(); i++ ){
 	
 	   	if ( (*i).is_selected() &&
 			 (*i).get_status() == a_status ){
@@ -1029,7 +1052,7 @@ sequence::decrement_selected(unsigned char a_status, unsigned char a_control )
 
 void sequence::clear_clipboard( void )
 {
-    m_list_clipboard->clear( );
+    m_list_clipboard.clear( );
 }
 
 void
@@ -1039,18 +1062,18 @@ sequence::copy_selected( void )
 
 	lock();
 
-    m_list_clipboard->clear( );
+    m_list_clipboard.clear( );
 
-    for ( i = m_list_event->begin(); i != m_list_event->end(); i++ ){
+    for ( i = m_list_event.begin(); i != m_list_event.end(); i++ ){
 
 	if ( (*i).is_selected() ){
-	    m_list_clipboard->push_back( (*i) );
+	    m_list_clipboard.push_back( (*i) );
 	}
     }
 
-    long first_tick = (*m_list_clipboard->begin()).get_timestamp();
+    long first_tick = (*m_list_clipboard.begin()).get_timestamp();
     
-    for ( i = m_list_clipboard->begin(); i != m_list_clipboard->end(); i++ ){
+    for ( i = m_list_clipboard.begin(); i != m_list_clipboard.end(); i++ ){
 
 	(*i).set_timestamp((*i).get_timestamp() - first_tick );
     }
@@ -1069,26 +1092,26 @@ sequence::paste_selected( long a_tick, int a_note )
 
 	lock();
 
-    for ( i = m_list_clipboard->begin(); i != m_list_clipboard->end(); i++ ){
+    for ( i = m_list_clipboard.begin(); i != m_list_clipboard.end(); i++ ){
 	(*i).set_timestamp((*i).get_timestamp() + a_tick );
     }
 
-    if ((*m_list_clipboard->begin()).is_note_on() ||
-	(*m_list_clipboard->begin()).is_note_off() ){
+    if ((*m_list_clipboard.begin()).is_note_on() ||
+	(*m_list_clipboard.begin()).is_note_off() ){
 	
-	for ( i = m_list_clipboard->begin(); i != m_list_clipboard->end(); i++ )
+	for ( i = m_list_clipboard.begin(); i != m_list_clipboard.end(); i++ )
 	    if ( (*i).get_note( ) > highest_note ) highest_note = (*i).get_note();
 
        
 
-	for ( i = m_list_clipboard->begin(); i != m_list_clipboard->end(); i++ ){
+	for ( i = m_list_clipboard.begin(); i != m_list_clipboard.end(); i++ ){
 
 	    (*i).set_note( (*i).get_note( ) - (highest_note - a_note) );
 	}
     }
 
-    m_list_event->merge( *m_list_clipboard );
-    m_list_event->sort();
+    m_list_event.merge( m_list_clipboard );
+    m_list_event.sort();
 
     verify_and_link();
 
@@ -1112,7 +1135,7 @@ sequence::change_event_data_range( long a_tick_s, long a_tick_f,
     unsigned char d0, d1;
     list<event>::iterator i;
 
-    for ( i = m_list_event->begin(); i != m_list_event->end(); i++ ){
+    for ( i = m_list_event.begin(); i != m_list_event.end(); i++ ){
 
 	/* initially false */
 	bool set = false;	
@@ -1195,13 +1218,13 @@ sequence::add_note( long a_tick, long a_length, int a_note )
 	event e;
 
 	e.set_status( EVENT_NOTE_ON );
-	e.set_data( a_note, 127 );
+	e.set_data( a_note, 100 );
 	e.set_timestamp( a_tick );
 
 	add_event( &e );
 
 	e.set_status( EVENT_NOTE_OFF );
-	e.set_data( a_note, 127 );
+	e.set_data( a_note, 100 );
 	e.set_timestamp( a_tick + a_length );
 
 	add_event( &e );
@@ -1330,7 +1353,7 @@ void
 sequence::clear_triggers( void )
 {
 	lock();
-	m_list_trigger->clear();
+	m_list_trigger.clear();
 	unlock();
 
 }
@@ -1351,24 +1374,24 @@ sequence::add_trigger( long a_tick, long a_length, bool a_state )
     e1.m_state = a_state;
     e2.m_state = ! a_state;
 
-    m_list_trigger->push_front( e1 );
-    m_list_trigger->push_front( e2 );
+    m_list_trigger.push_front( e1 );
+    m_list_trigger.push_front( e2 );
 
-    m_list_trigger->sort();
+    m_list_trigger.sort();
 
-    list<trigger>::iterator i = m_list_trigger->begin();
-    list<trigger>::iterator j = m_list_trigger->begin();	 
+    list<trigger>::iterator i = m_list_trigger.begin();
+    list<trigger>::iterator j = m_list_trigger.begin();	 
     j++;
 
-    list<trigger>::iterator first = m_list_trigger->begin();
-    list<trigger>::iterator last = m_list_trigger->begin();
+    list<trigger>::iterator first = m_list_trigger.begin();
+    list<trigger>::iterator last = m_list_trigger.begin();
 
     bool contain = false;
 
     /* range on */
     if ( a_state ){
 
-	while(  j != m_list_trigger->end() ){
+	while(  j != m_list_trigger.end() ){
 	
 	    /* both on */
 	    if ( (*i).m_state == true && (*j).m_state == true ){
@@ -1388,18 +1411,18 @@ sequence::add_trigger( long a_tick, long a_length, bool a_state )
     /* range off */
     else {
 
-	if ( m_list_trigger->size() == 2 ){
+	if ( m_list_trigger.size() == 2 ){
 	    
-	    m_list_trigger->clear();
+	    m_list_trigger.clear();
 	    unlock();
 	    return;
 	}
 
-	while(  j != m_list_trigger->end() ){
+	while(  j != m_list_trigger.end() ){
 	    
 	    /* both off */
 	    if ( (*i).m_state == false && (*j).m_state == false ){
-		first = j; last = m_list_trigger->end();
+		first = j; last = m_list_trigger.end();
 		contain = true;
 	    }
 	    
@@ -1414,24 +1437,24 @@ sequence::add_trigger( long a_tick, long a_length, bool a_state )
     }
 
     /* we might get an OFF at the begining */
-    if ( (*m_list_trigger->begin()).m_state == false ){
-	first = m_list_trigger->begin(); contain = true;
+    if ( (*m_list_trigger.begin()).m_state == false ){
+	first = m_list_trigger.begin(); contain = true;
     }
 
     if ( contain )
-	m_list_trigger->erase(first,last);
+	m_list_trigger.erase(first,last);
 
-    if ( (*m_list_trigger->begin()).m_state == false )
-	m_list_trigger->pop_front();
+    if ( (*m_list_trigger.begin()).m_state == false )
+	m_list_trigger.pop_front();
 
     validate_triggers( );
 
     /* elements should always be in pairs,
        if not clear, or the drawing routines will hang,
        this should really never be called */
-    if ( m_list_trigger->size() % 2 != 0 ){
+    if ( m_list_trigger.size() % 2 != 0 ){
 	
-	m_list_trigger->clear();
+	m_list_trigger.clear();
     }
 
     unlock();
@@ -1441,22 +1464,22 @@ void
 sequence::validate_triggers( void  )
 {
 
-    list<trigger>::iterator i = m_list_trigger->begin();
-    list<trigger>::iterator j = m_list_trigger->begin();
+    list<trigger>::iterator i = m_list_trigger.begin();
+    list<trigger>::iterator j = m_list_trigger.begin();
 
 	lock();
 
     j++;
     
-    while(  j != m_list_trigger->end() ){
+    while(  j != m_list_trigger.end() ){
 	
  	if ( ((*i).m_state != (*j).m_state  &&
 	      (*i).m_tick + 1 >= (*j).m_tick )){
 	    
- 	    m_list_trigger->erase(i);
- 	    m_list_trigger->erase(j);
+ 	    m_list_trigger.erase(i);
+ 	    m_list_trigger.erase(j);
 	    
-	    i = j = m_list_trigger->begin(); j++;
+	    i = j = m_list_trigger.begin(); j++;
 	    
  	} else {
 	    
@@ -1483,9 +1506,9 @@ sequence::move_triggers( long a_start_tick,
 	first_tick += a_distance;
     }
 
-    list<trigger>::iterator i = m_list_trigger->begin(); 
+    list<trigger>::iterator i = m_list_trigger.begin(); 
 
-    while(  i != m_list_trigger->end() ){
+    while(  i != m_list_trigger.end() ){
 
 	if ( (*i).m_tick >= first_tick ){
 
@@ -1521,7 +1544,7 @@ sequence::copy_triggers( long a_start_tick,
 		   true );
     
 	
-    list<trigger> m_list_trigger_copy = *m_list_trigger;
+    list<trigger> m_list_trigger_copy = m_list_trigger;
  
     list<trigger>::iterator i = m_list_trigger_copy.begin(); 
     trigger e1;
@@ -1534,7 +1557,7 @@ sequence::copy_triggers( long a_start_tick,
 	    e1.m_tick  = (*i).m_tick - a_distance;
 	    e1.m_state = (*i).m_state;
 
-	    m_list_trigger->push_front( e1 );
+	    m_list_trigger.push_front( e1 );
 	}
 	
 	i++;
@@ -1546,11 +1569,11 @@ sequence::copy_triggers( long a_start_tick,
 	e1.m_state = false;
 	e1.m_tick = first_tick + a_distance;
 
-	m_list_trigger->push_front( e1 );
+	m_list_trigger.push_front( e1 );
     }
 	
 
-    m_list_trigger->sort();
+    m_list_trigger.sort();
     validate_triggers( );
 
 	unlock();
@@ -1565,8 +1588,8 @@ sequence::get_max_trigger( void )
 
     long ret;
 
-    if ( m_list_trigger->size() > 0 )
-	ret = m_list_trigger->back().m_tick;
+    if ( m_list_trigger.size() > 0 )
+	ret = m_list_trigger.back().m_tick;
     else
 	ret = 0;
 
@@ -1583,7 +1606,7 @@ sequence::get_trigger_state( long a_tick )
     bool ret = false;
     list<trigger>::iterator i;
 
-    for ( i = m_list_trigger->begin(); i != m_list_trigger->end(); i++ ){
+    for ( i = m_list_trigger.begin(); i != m_list_trigger.end(); i++ ){
 
 	if ( (*i).m_tick <= a_tick )
 	    ret = (*i).m_state;
@@ -1605,7 +1628,7 @@ sequence::reset_draw_marker( void )
 {
 	lock();
 
-    m_iterator_draw = m_list_event->begin();
+    m_iterator_draw = m_list_event.begin();
 
 	unlock();
 }
@@ -1615,7 +1638,7 @@ sequence::reset_draw_trigger_marker( void )
 {
 	lock();
 
-    m_iterator_draw_trigger = m_list_trigger->begin();
+    m_iterator_draw_trigger = m_list_trigger.begin();
 
 	unlock();
 }
@@ -1629,7 +1652,7 @@ sequence::get_lowest_note_event( void )
     int ret = 127;
     list<event>::iterator i;
 
-    for ( i = m_list_event->begin(); i != m_list_event->end(); i++ ){
+    for ( i = m_list_event.begin(); i != m_list_event.end(); i++ ){
 
 	if ( (*i).is_note_on() || (*i).is_note_off() )
 	    if ( (*i).get_note() < ret )
@@ -1651,7 +1674,7 @@ sequence::get_highest_note_event( void )
     int ret = 0;
     list<event>::iterator i;
 
-    for ( i = m_list_event->begin(); i != m_list_event->end(); i++ ){
+    for ( i = m_list_event.begin(); i != m_list_event.end(); i++ ){
 
 	if ( (*i).is_note_on() || (*i).is_note_off() )
 	    if ( (*i).get_note() > ret )
@@ -1672,8 +1695,9 @@ sequence::get_next_note_event( long *a_tick_s,
 {
 
     draw_type ret = DRAW_FIN;
+    *a_tick_f = 0;
 
-    while (  m_iterator_draw  != m_list_event->end() )
+    while (  m_iterator_draw  != m_list_event.end() )
     {
 	*a_tick_s   = (*m_iterator_draw).get_timestamp();
 	*a_note     = (*m_iterator_draw).get_note();
@@ -1714,6 +1738,26 @@ sequence::get_next_note_event( long *a_tick_s,
 }
 
 
+bool
+sequence::get_next_event( unsigned char *a_status,
+                          unsigned char *a_cc)
+{
+    unsigned char j;
+    
+    while (  m_iterator_draw  != m_list_event.end() ){
+        
+        *a_status = (*m_iterator_draw).get_status();
+        (*m_iterator_draw).get_data( a_cc, &j );
+        
+		/* we have a good one */
+		/* update and return */
+		m_iterator_draw++;
+		return true;
+    }
+    return false;
+}
+
+
 bool 
 sequence::get_next_event( unsigned char a_status,
 			  unsigned char a_cc,
@@ -1722,7 +1766,7 @@ sequence::get_next_event( unsigned char a_status,
 			  unsigned char *a_D1,
 			  bool *a_selected )
 {
-    while (  m_iterator_draw  != m_list_event->end() ){ 
+    while (  m_iterator_draw  != m_list_event.end() ){ 
 	
 	/* note on, so its linked */
 	if( (*m_iterator_draw).get_status() == a_status ){
@@ -1752,7 +1796,7 @@ sequence::get_next_event( unsigned char a_status,
 bool 
 sequence::get_next_trigger( long *a_tick_on, long *a_tick_off )
 {
-    while (  m_iterator_draw_trigger  != m_list_trigger->end() ){ 
+    while (  m_iterator_draw_trigger  != m_list_trigger.end() ){ 
 
 	*a_tick_on  = (*m_iterator_draw_trigger).m_tick;
 	m_iterator_draw_trigger++;
@@ -1770,7 +1814,9 @@ sequence::remove_all( void )
 {
     lock();
 
-    m_list_event->clear();
+	
+
+    m_list_event.clear();
 
     unlock();
 
@@ -1784,8 +1830,8 @@ sequence::operator= (const sequence& a_rhs)
     /* dont copy to self */
     if (this != &a_rhs){
 	
-	(*m_list_event)   = (*a_rhs.m_list_event);
-	(*m_list_trigger)   = (*a_rhs.m_list_trigger);
+	m_list_event   = a_rhs.m_list_event;
+	m_list_trigger   = a_rhs.m_list_trigger;
 
 	m_midi_channel = a_rhs.m_midi_channel;
 	m_masterbus    = a_rhs.m_masterbus;
@@ -1870,15 +1916,15 @@ void
 sequence::set_length( long a_len )
 {
     lock();
-
-    if ( a_len > c_ppqn * 64 )
-      a_len = c_ppqn * 64;
  
     bool was_playing = get_playing();
 
     /* turn everything off */
     set_playing( false );
 
+    if ( a_len < (c_ppqn / 4) )
+        a_len = (c_ppqn /4);;
+    
     m_length = a_len;
 
     verify_and_link();
@@ -2008,9 +2054,9 @@ sequence::print()
 {
     printf("[%s]\n", m_name.c_str()  );
 
-    for( list<event>::iterator i = m_list_event->begin(); i != m_list_event->end(); i++ )
+    for( list<event>::iterator i = m_list_event.begin(); i != m_list_event.end(); i++ )
 	(*i).print();
-    printf("events[%d]\n\n",m_list_event->size());
+    printf("events[%d]\n\n",m_list_event.size());
 
 }
 
@@ -2080,6 +2126,201 @@ sequence::off_playing_notes()
 
 
 
+
+
+
+
+/* change */
+void 
+sequence::select_events( unsigned char a_status, unsigned char a_cc, bool a_inverse )
+{
+    lock();
+
+    unsigned char d0, d1;
+    list<event>::iterator i;
+
+    for ( i = m_list_event.begin(); i != m_list_event.end(); i++ ){
+
+	/* initially false */
+	bool set = false;	
+	(*i).get_data( &d0, &d1 );
+	
+	/* correct status and not CC */
+	if ( a_status != EVENT_CONTROL_CHANGE &&
+	     (*i).get_status() == a_status )
+	    set = true;
+	
+	/* correct status and correct cc */
+	if ( a_status == EVENT_CONTROL_CHANGE &&
+	     (*i).get_status() == a_status &&
+	     d0 == a_cc )
+	    set = true;
+	
+        if ( set ){
+
+            if ( a_inverse ){
+                if ( !(*i).is_selected( ) )
+                    (*i).select( );
+                else
+                    (*i).unselect( );
+                    
+            }
+            else 
+                (*i).select( );
+	}	    
+    }
+
+    unlock();
+}
+
+void
+sequence::transpose_notes( int a_steps, int a_scale )
+{
+    event e;
+
+    //unselect( );
+    list<event> transposed_events;
+
+    lock();
+
+    list<event>::iterator i;
+
+    const int *transpose_table = NULL;
+
+    if ( a_steps < 0 ){
+        transpose_table = &c_scales_transpose_dn[a_scale][0];
+        a_steps *= -1;
+    }
+    else {
+        transpose_table = &c_scales_transpose_up[a_scale][0];
+    }
+
+    for ( i = m_list_event.begin(); i != m_list_event.end(); i++ ){
+	
+	/* is it being moved ? */
+	if ( ((*i).get_status() ==  EVENT_NOTE_ON ||
+             (*i).get_status() ==  EVENT_NOTE_OFF) &&
+             (*i).is_selected() ){
+
+            e = (*i);
+
+            int  note = e.get_note();
+
+            bool off_scale = false;
+            if (  transpose_table[note % 12] == 0 ){
+                off_scale = true;
+                note -= 1;
+            }
+
+            for( int x=0; x<a_steps; ++x )
+                note += transpose_table[note % 12];
+
+            if ( off_scale )
+                note += 1;
+            
+            e.set_note( note );
+   
+            transposed_events.push_front(e);
+
+            (*i).select();
+	}
+    }
+
+    remove_selected();
+    transposed_events.sort();
+    m_list_event.merge( transposed_events);
+    
+   
+    verify_and_link();
+
+    unlock();
+
+    
+}
+
+
+
+// NOT DELETING THE ENDS, NOT SELECTED.
+void
+sequence::quanize_events( unsigned char a_status, unsigned char a_cc,
+                          long a_snap_tick,  int a_divide, bool a_linked )
+{
+    event e,f;
+
+    lock();
+
+    unsigned char d0, d1;
+    list<event>::iterator i;
+
+    //unselect( );
+    list<event> quantized_events;
+
+    for ( i = m_list_event.begin(); i != m_list_event.end(); i++ ){
+
+        /* initially false */
+	bool set = false;	
+	(*i).get_data( &d0, &d1 );
+	
+	/* correct status and not CC */
+	if ( a_status != EVENT_CONTROL_CHANGE &&
+	     (*i).get_status() == a_status )
+	    set = true;
+	
+	/* correct status and correct cc */
+	if ( a_status == EVENT_CONTROL_CHANGE &&
+	     (*i).get_status() == a_status &&
+	     d0 == a_cc )
+	    set = true;
+
+        if( !(*i).is_selected() )
+            set = false;
+	
+        if ( set ){
+
+            /* copy event */
+	    e = (*i);
+            (*i).select();
+
+            long timestamp = e.get_timestamp();
+            long timestamp_remander = (timestamp % a_snap_tick);
+            long timestamp_delta = 0;
+            
+            if ( timestamp_remander < a_snap_tick/2 ){
+                timestamp_delta = - (timestamp_remander / a_divide );
+            }
+            else {
+                timestamp_delta = (a_snap_tick - timestamp_remander) / a_divide;
+            }
+            
+            e.set_timestamp( e.get_timestamp() + timestamp_delta );
+            quantized_events.push_front(e);
+            
+            if ( (*i).is_linked() && a_linked ){
+                
+                f = *(*i).get_linked();
+                (*i).get_linked()->select();
+                
+                f.set_timestamp( f.get_timestamp() + timestamp_delta );
+                quantized_events.push_front(f);
+            }
+        }
+
+    }
+
+    remove_selected();
+    quantized_events.sort();
+    m_list_event.merge(quantized_events);
+    verify_and_link();
+
+    unlock();
+
+}
+
+
+
+
+
+
 void 
 addListVar( list<char> *a_list, long a_var )
 {
@@ -2147,7 +2388,7 @@ sequence::fill_list( list<char> *a_list, int a_pos )
     long timestamp = 0, delta_time = 0, prev_timestamp = 0;
     list<event>::iterator i;
 
-    for ( i = m_list_event->begin(); i != m_list_event->end(); i++ ){
+    for ( i = m_list_event.begin(); i != m_list_event.end(); i++ ){
 
 	event e = (*i);
 	timestamp = e.get_timestamp();
@@ -2191,8 +2432,8 @@ sequence::fill_list( list<char> *a_list, int a_pos )
 	}
     }
 
-    int num_triggers = m_list_trigger->size();
-    list<trigger>::iterator t = m_list_trigger->begin();
+    int num_triggers = m_list_trigger.size();
+    list<trigger>::iterator t = m_list_trigger.begin();
 
     addListVar( a_list, 0 );
     a_list->push_front( 0xFF );
@@ -2246,12 +2487,15 @@ sequence::fill_list( list<char> *a_list, int a_pos )
 
 
 
+
+
+
 //     list<char> triggers;
 
 //     /* triggers */
 
 //     list<trigger>::iterator t;
-//     for ( t = m_list_trigger->begin(); t != m_list_trigger->end(); t++ ){
+//     for ( t = m_list_trigger.begin(); t != m_list_trigger.end(); t++ ){
 
 // 	addLongList( &triggers, (*t).m_tick );
 // 	printf ( "[%ld]\n", (*t).m_tick );
