@@ -30,6 +30,7 @@ class sequence;
 
 #include <string>
 #include <list>
+#include <stack>
 
 enum draw_type {
     
@@ -68,11 +69,12 @@ class sequence
  private:
 
     /* holds the events */
-    list<event> *m_list_event;
-    list<event> *m_list_clipboard;
+    list<event> m_list_event;
+    list<event> m_list_clipboard;
 
-    list<trigger> *m_list_trigger;
+    list<trigger> m_list_trigger;
  
+	stack< list<event> > m_list_undo;
 
     /* markers */
     list<event>::iterator m_iterator_play;
@@ -143,11 +145,16 @@ class sequence
     void lock();
     void unlock();
 
+
  public:
 
     sequence( );
     ~sequence( );
 
+
+	void push_undo( void );
+	void pop_undo( void );
+
     //
     //  Gets and Sets
     //
@@ -347,6 +354,9 @@ class sequence
 			 unsigned char *a_D1,
 			 bool *a_selected );
 
+    bool get_next_event( unsigned char *a_status,
+                         unsigned char *a_cc);
+
     bool get_next_trigger( long *a_tick_on,
 			   long *a_tick_off );
 
@@ -354,6 +364,9 @@ class sequence
 
     void fill_list( list<char> *a_list, int a_pos );
 
+    void select_events( unsigned char a_status, unsigned char a_cc, bool a_inverse = false );
+    void quanize_events( unsigned char a_status, unsigned char a_cc, long a_snap_tick, int a_divide, bool a_linked = false );
+    void transpose_notes( int a_steps, int a_scale );
 };
 
 #endif
